@@ -7,7 +7,7 @@ the imdb.IMDb function will return an instance of this class when
 called with the 'accessSystem' argument set to "http" or "web"
 or "html" (this is the default).
 
-Copyright 2004, 2005 Davide Alberani <da@erlug.linux.it>
+Copyright 2004-2006 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -24,10 +24,11 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
-import urllib
+from urllib import FancyURLopener, quote_plus
+from codecs import lookup
 
 from imdb import IMDbBase
-from imdb._exceptions import IMDbDataAccessError
+from imdb._exceptions import IMDbDataAccessError, IMDbParserError
 from movieParser import movie_parser, plot_parser, movie_awards_parser, \
                         taglines_parser, keywords_parser, \
                         alternateversions_parser, crazycredits_parser, \
@@ -35,13 +36,18 @@ from movieParser import movie_parser, plot_parser, movie_awards_parser, \
                         releasedates_parser, ratings_parser, \
                         officialsites_parser, connections_parser, \
                         tech_parser, locations_parser, soundtrack_parser, \
-                        dvd_parser
+                        dvd_parser, rec_parser, externalrev_parser, \
+                        newsgrouprev_parser, misclinks_parser, \
+                        soundclips_parser, videoclips_parser, news_parser, \
+                        photosites_parser, amazonrev_parser, guests_parser, \
+                        business_parser, sales_parser
 from searchMovieParser import search_movie_parser
 from personParser import maindetails_parser, bio_parser, \
                         otherworks_parser, person_awards_parser, \
                         person_officialsites_parser, publicity_parser, \
                         agent_parser
 from searchPersonParser import search_person_parser
+from utils import ParserBase
 
 
 # Misc URLs
@@ -55,10 +61,10 @@ _cookie_id = 'boM2bYxz9MCsOnH9gZ0S9QHs12NWrNdApxsls1Vb5/NGrNdjcHx3dUas10UASoAjVE
 _cookie_uu = '3M3AXsquTU5Gur/Svik+ewflPm5Rk2ieY3BIPlLjyK3C0Dp9F8UoPgbTyKiGtZp4x1X+uAUGKD7BM2g+dVd8eqEzDErCoYvdcvGLvVLAen1y08hNQtALjVKAe+1hM8g9QbNonlG1/t4S82ieUsBbrSIQbq1yhV6tZ6ArvSbA7rgHc8n5AdReyAmDaJ5Wm/ee3VDoCnGj/LlBs2ieUZNorhHDKK5Q=='
 
 
-class IMDbURLopener(urllib.FancyURLopener):
+class IMDbURLopener(FancyURLopener):
     """Fetch web pages and handle errors."""
     def __init__(self, *args, **kwargs):
-        urllib.FancyURLopener.__init__(self, *args, **kwargs)
+        FancyURLopener.__init__(self, *args, **kwargs)
         # XXX: IMDb's web server doesn't like urllib-based programs,
         #      so lets fake to be Mozilla.
         #      Wow!  I'm shocked by my total lack of ethic! <g>
@@ -87,7 +93,8 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     urlOpener = IMDbURLopener()
 
-    def __init__(self, isThin=0, *arguments, **keywords):
+    def __init__(self, isThin=0, adultSearch=1, proxy=-1,
+                *arguments, **keywords):
         """Initialize the access system."""
         IMDbBase.__init__(self, *arguments, **keywords)
         # When isThin is set, we're parsing the "maindetails" page
@@ -97,19 +104,29 @@ class IMDbHTTPAccessSystem(IMDbBase):
         if isThin:
             self.accessSystem = 'httpThin'
             movie_parser.mdparse = 1
-            from utils import ParserBase
             if self._defModFunct is None:
                 ParserBase.getRefs = 0
                 from imdb.utils import modNull
                 self._defModFunct = modNull
+        else:
+            movie_parser.mdparse = 0
+            ParserBase.getRefs = 1
+        self.do_adult_search(adultSearch)
+        if proxy != -1: self.set_proxy(proxy)
 
     def _normalize_movieID(self, movieID):
         """Normalize the given movieID."""
-        return '%07d' % int(movieID)
+        try:
+            return '%07d' % int(movieID)
+        except ValueError, e:
+            raise IMDbParserError, 'invalid movieID "%s": %s' % (movieID, e)
 
     def _normalize_personID(self, personID):
         """Normalize the given personID."""
-        return '%07d' % int(personID)
+        try:
+            return '%07d' % int(personID)
+        except ValueError, e:
+            raise IMDbParserError, 'invalid personID "%s": %s' % (personID, e)
 
     def get_imdbMovieID(self, movieID):
         """Translate a movieID in an imdbID; in this implementation
@@ -159,9 +176,25 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def _retrieve(self, url):
         """Retrieve the given URL."""
+        encode = 'latin_1'
         try:
             uopener = self.urlOpener.open(url)
             content = uopener.read()
+            server_encode = uopener.info().getparam('charset')
+            # look at the content-type HTML meta tag.
+            if server_encode is None and content:
+                first_bytes = content[:512]
+                begin_h = first_bytes.find('text/html; charset=')
+                if begin_h != -1:
+                    end_h = first_bytes[19+begin_h:].find('"')
+                    if end_h != -1:
+                        server_encode = first_bytes[19+begin_h:19+begin_h+end_h]
+            if server_encode:
+                try:
+                    if lookup(server_encode):
+                        encode = server_encode
+                except (LookupError, ValueError, TypeError):
+                    pass
             uopener.close()
             self.urlOpener.close()
         except IOError, e:
@@ -169,13 +202,14 @@ class IMDbHTTPAccessSystem(IMDbBase):
                                         'errmsg': str(e.strerror),
                                         'url': url,
                                         'proxy': self.get_proxy()}
-        return content
+        return unicode(content, encode, 'replace')
 
     def _search_movie(self, title, results):
         # The URL of the query.
         # XXX: To retrieve the complete results list:
         #      params = urllib.urlencode({'more': 'tt', 'q': title})
-        params = urllib.urlencode({'tt': 'on', 'mx': str(results), 'q': title})
+        ##params = urllib.urlencode({'tt': 'on','mx': str(results),'q': title})
+        params = 'q=%s&tt=on&mx=%s' % (quote_plus(title), str(results))
         cont = self._retrieve(imdbURL_search % params)
         return search_movie_parser.parse(cont, results=results)['data']
 
@@ -244,7 +278,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def get_movie_business(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'business')
-        return tech_parser.parse(cont)
+        return business_parser.parse(cont)
 
     def get_movie_literature(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'literature')
@@ -253,7 +287,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
     def get_movie_locations(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'locations')
         return locations_parser.parse(cont)
-    
+
     def get_movie_soundtrack(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'soundtrack')
         return soundtrack_parser.parse(cont)
@@ -261,12 +295,57 @@ class IMDbHTTPAccessSystem(IMDbBase):
     def get_movie_dvd(self, movieID):
         cont = self._retrieve(imdbURL_movie % movieID + 'dvd')
         return dvd_parser.parse(cont)
-    
+
+    def get_movie_recommendations(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'recommendations')
+        return rec_parser.parse(cont)
+
+    def get_movie_external_reviews(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'externalreviews')
+        return externalrev_parser.parse(cont)
+
+    def get_movie_newsgroup_reviews(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'newsgroupreviews')
+        return newsgrouprev_parser.parse(cont)
+
+    def get_movie_misc_sites(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'miscsites')
+        return misclinks_parser.parse(cont)
+
+    def get_movie_sound_clips(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'soundsites')
+        return soundclips_parser.parse(cont)
+
+    def get_movie_video_clips(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'videosites')
+        return videoclips_parser.parse(cont)
+
+    def get_movie_photo_sites(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'photosites')
+        return photosites_parser.parse(cont)
+
+    def get_movie_news(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'news')
+        return news_parser.parse(cont)
+
+    def get_movie_amazon_reviews(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'amazon')
+        return amazonrev_parser.parse(cont)
+
+    def get_movie_guests(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'epcast')
+        return guests_parser.parse(cont)
+
+    def get_movie_merchandising_links(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'sales')
+        return sales_parser.parse(cont)
+
     def _search_person(self, name, results):
         # The URL of the query.
         # XXX: To retrieve the complete results list:
         #      params = urllib.urlencode({'more': 'nm', 'q': name})
-        params = urllib.urlencode({'nm': 'on', 'mx': str(results), 'q': name})
+        ##params = urllib.urlencode({'nm': 'on', 'mx': str(results), 'q': name})
+        params = 'q=%s&nm=on&mx=%s' % (quote_plus(name), str(results))
         cont = self._retrieve(imdbURL_search % params)
         return search_person_parser.parse(cont, results=results)['data']
 
@@ -303,4 +382,8 @@ class IMDbHTTPAccessSystem(IMDbBase):
         cont = self._retrieve(imdbURL_person % personID + 'officialsites')
         return person_officialsites_parser.parse(cont)
 
+    def get_person_news(self, personID):
+        cont = self._retrieve(imdbURL_person % personID + 'news')
+        return news_parser.parse(cont)
+
 
