@@ -6,7 +6,7 @@ IMDb's data for mobile systems.
 the imdb.IMDb function will return an instance of this class when
 called with the 'accessSystem' argument set to "mobile".
 
-Copyright 2005 Davide Alberani <da@erlug.linux.it>
+Copyright 2005-2006 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -24,62 +24,35 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
 import re, urllib
-from htmlentitydefs import entitydefs
-from sgmllib import entityref, charref
 
 from imdb.Movie import Movie
 from imdb.Person import Person
-from imdb.utils import analyze_title, analyze_name, \
-                        canonicalTitle, canonicalName
+from imdb.utils import analyze_title, analyze_name, canonicalName
 from imdb._exceptions import IMDbDataAccessError
 from imdb.parser.http import IMDbHTTPAccessSystem, imdbURL_search, \
                                 imdbURL_movie, imdbURL_person
+from imdb.parser.http.utils import subXMLRefs, subSGMLRefs
 
 # XXX NOTE: the first version of this module was heavily based on
 #           regular expressions.  This new version replace regexps with
-#           find() strings methods; despite being less flexible, it
+#           find() strings' method calls; despite being less flexible, it
 #           seems to be at least as fast and, hopefully, much more
-#           lightweight.
+#           lightweight.  Yes: the regexp-based version was too heavyweight
+#           for systems with very limited CPU power and memory footprint.
 
 # To strip spaces.
 re_spaces = re.compile(r'\s+')
+re_spacessub = re_spaces.sub
 # Strip html.
 re_unhtml = re.compile(r'<.+?>')
+re_unhtmlsub = re_unhtml.sub
 # imdb person or movie ids.
 re_imdbID = re.compile(r'(?<=nm|tt)([0-9]{7})\b')
 
-# Here to handle non-breaking spaces.
-entitydefs['nbsp'] = ' '
-
-
-def _replRef(match):
-    """Replace the matched html/sgml entity and reference."""
-    ret = match.group()[1:-1]
-    ret = entitydefs.get(ret, ret)
-    if ret[0] == '#':
-        try:
-            ret = chr(int(ret[1:]))
-            if ret == '\xa0': ret = ' '
-        except (ValueError, TypeError, OverflowError):
-            try:
-                ret = unichr(int(ret[1:])).encode('utf-8')
-            except (ValueError, TypeError, OverflowError):
-                pass
-            pass
-    return ret
-
-
-def _subRefs(s):
-    """Return the given html string with entity and char references
-    replaced."""
-    s = entityref.sub(_replRef, s)
-    s = charref.sub(_replRef, s)
-    return s
-
 
 def _unHtml(s):
-    """Return a string without tags."""
-    return re_spaces.sub(' ', re_unhtml.sub('', s)).strip()
+    """Return a string without tags and no multiple spaces."""
+    return subSGMLRefs(re_spacessub(' ', re_unhtmlsub('', s)).strip())
 
 
 _ltypes = (type([]), type(()))
@@ -100,8 +73,8 @@ def _getTagWith(s, cont):
 
 
 def _findBetween(s, begins, ends, beginindx=0):
-    """Return the list of string from the s string which included
-    between the begins and ends strings."""
+    """Return the list of strings from the 's' string which are included
+    between the 'begins' and 'ends' strings."""
     lres = []
     #if endindx is None: endindx = len(s)
     bi = s.find(begins, beginindx)
@@ -118,7 +91,7 @@ def _findBetween(s, begins, ends, beginindx=0):
             match = s[bi+lbegins:ei]
             lres.append(match)
             lres += _findBetween(s, begins, ends, ei)
-            #if maxRes > 0 and len(lres) >= maxRes: return lres
+            ##if maxRes > 0 and len(lres) >= maxRes: return lres
     return lres
 
 
@@ -136,59 +109,71 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         """Retrieve an html page and normalize it."""
         cont = IMDbHTTPAccessSystem._retrieve(self, url)
         cont = re_spaces.sub(' ', cont)
-        return _subRefs(cont)
+        cont = subXMLRefs(cont)
+        return cont
 
-    def _getPersons(self, s, sep='<br>', hasCr=0):
+    def _getPersons(self, s, sep='<br>', hasCr=0, aonly=0):
         """Return a list of Person objects, from the string s; items
-        are separated by the sep string; if hasCr is set, the
-        currentRole of a person is searched."""
+        are assumed to be separated by the sep string; if hasCr is set,
+        the currentRole of a person is searched."""
         names = s.split(sep)
         pl = []
+        plappend = pl.append
+        counter = 1
+        currentRole = u''
         for name in names:
-            notes = ''
-            currentRole = ''
+            notes = u''
+            currentRole = u''
             fpi = name.find(' (')
             if fpi != -1:
                 fpe = name.rfind(')')
                 if fpe > fpi:
                     notes = _unHtml(name[fpi:fpe+1])
                     name = name[:fpi] + name[fpe+1:]
-                    name = name.replace('&', '')
+                    lampi = name.rfind('&')
+                    if lampi != -1 and name[lampi+1:].isspace():
+                        name = name[:lampi].rstrip()
             if hasCr:
                 name = name.split(' .... ')
                 if len(name) > 1:
                     currentRole = _unHtml(name[1])
                 name = name[0]
             pid = re_imdbID.findall(name)
+            if aonly:
+                stripped = _findBetween(name, '>', '</a>')
+                if len(stripped) == 1: name = stripped[0]
             name = _unHtml(name)
             if not (pid and name): continue
-            pl.append(Person(personID=pid[0], name=canonicalName(name),
+            plappend(Person(personID=str(pid[0]), name=name,
                             currentRole=currentRole, notes=notes,
                             accessSystem=self.accessSystem,
-                            modFunct=self._defModFunct))
+                            modFunct=self._defModFunct, billingPos=counter))
+            counter += 1
         return pl
 
     def _search_movie(self, title, results):
-        params = urllib.urlencode({'tt': 'on', 'mx': str(results), 'q': title})
+        ##params = urllib.urlencode({'tt': 'on','mx': str(results),'q': title})
+        params = 'q=%s&tt=on&mx=%s' % (urllib.quote_plus(title), str(results))
         cont = self._mretrieve(imdbURL_search % params)
         title = _findBetween(cont, '<title>', '</title>')
         res = []
         if not title: return res
-        if title[0].lower().find('imdb title search') == -1:
+        tl = title[0].lower()
+        if not tl.startswith('imdb title'):
             # XXX: a direct hit!
             title = _unHtml(title[0])
             midtag = _getTagWith(cont, 'name="arg"')
             mid = None
             if midtag: mid = _findBetween(midtag[0], 'value="', '"')
             if not (mid and title): return res
-            res[:] = [(mid[0], analyze_title(title, canonical=1))]
+            res[:] = [(str(mid[0]), analyze_title(title, canonical=1))]
         else:
             lis = _findBetween(cont, '<li>', ['</li>', '<br>'])
             for li in lis:
                 imdbid = re_imdbID.findall(li)
                 mtitle = _unHtml(li)
                 if not (imdbid and mtitle): continue
-                res.append((imdbid[0], analyze_title(mtitle, canonical=1)))
+                res.append((str(imdbid[0]), analyze_title(mtitle, canonical=1)))
         return res
 
     def get_movie_main(self, movieID):
@@ -201,16 +186,25 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         d = analyze_title(title, canonical=1)
         direct = _findBetween(cont, 'Directed by</b><br>', '<br> <br>')
         if direct:
-            dirs = self._getPersons(direct[0])
+            dirs = self._getPersons(direct[0], aonly=1)
             if dirs: d['director'] = dirs
         writers = _findBetween(cont, 'Writing credits</b>', '<br> <br>')
         if writers:
-            ws = self._getPersons(writers[0])
+            ws = self._getPersons(writers[0], aonly=1)
             if ws: d['writer'] = ws
         cvurl = _getTagWith(cont, 'alt="cover"')
         if cvurl:
             cvurl = _findBetween(cvurl[0], 'src="', '"')
             if cvurl: d['cover url'] = cvurl[0]
+        if not d.has_key('cover url'):
+            # Cover, the new style.
+            short_title = d.get('title', u'')
+            if short_title:
+                cvurl = _getTagWith(cont, 'alt="%s"' % short_title)
+                cvurl[:] = [x for x in cvurl if x[0:4] == '<img']
+                if cvurl:
+                    cvurl = _findBetween(cvurl[0], 'src="', '"')
+                    if cvurl: d['cover url'] = cvurl[0]
         genres = _findBetween(cont, 'href="/Sections/Genres/', '/')
         if genres: d['genres'] = genres
         ur = _findBetween(cont, 'User Rating:</b>', ' votes)')
@@ -220,25 +214,48 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                 teni = rat[0].find('/10')
                 if teni != -1:
                     rat = rat[0][:teni]
-                    d['rating'] = rat.strip()
+                    try:
+                        rat = float(rat.strip())
+                        d['rating'] = rat
+                    except ValueError:
+                        pass
             vi = ur[0].rfind('(')
             if vi != -1 and ur[0][vi:].find('await') == -1:
-                d['votes'] = ur[0][vi+1:].replace(',', '').strip()
+                try:
+                    votes = int(ur[0][vi+1:].replace(',', '').strip())
+                    d['votes'] = votes
+                except ValueError:
+                    pass
         top250 = _findBetween(cont, 'href="/top_250_films"', '</a>')
         if top250:
             fn = top250[0].rfind('#')
             if fn != -1:
-                d['top 250 rank'] = top250[0][fn+1:]
+                try:
+                    td = int(top250[0][fn+1:])
+                    d['top 250 rank'] = td
+                except ValueError:
+                    pass
         castdata = _findBetween(cont, 'Cast overview', '</table>')
+        if not castdata:
+            castdata = _findBetween(cont, 'Credited cast', '</table>')
+        if not castdata:
+            castdata = _findBetween(cont, 'Complete credited cast', '</table>')
         if castdata:
-            fl = castdata[0].find('href=')
-            if fl != -1: castdata[0] = '< a' + castdata[0][fl:]
-            cast = self._getPersons(castdata[0], sep='</tr><tr>', hasCr=1)
+            castdata = castdata[0]
+            fl = castdata.find('href=')
+            if fl != -1: castdata = '< a' + castdata[fl:]
+            smib = castdata.find('<tr><td align="center" colspan="3"><small>')
+            if smib != -1:
+                smie = castdata.rfind('</small></td></tr>')
+                if smie != -1:
+                    castdata = castdata[:smib].strip() + \
+                                castdata[smie+18:].strip()
+            cast = self._getPersons(castdata, sep='</tr><tr>', hasCr=1)
             if cast: d['cast'] = cast
         # FIXME: doesn't catch "complete title", which is not
         #        included in <i> tags.
         #        See "Gehr Nany Fgbevrf 11", movieID: 0282910
-        akas = _findBetween(cont, '<i class="transl">', '<br>')
+        akas = _findBetween(cont, '<i class="transl">', '<br')
         if akas:
             akas = [_unHtml(x).replace(' (','::(', 1).replace(' [','::[')
                     for x in akas]
@@ -250,27 +267,31 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             rt = [x.strip().replace(' min', '')
                     for x in runtimes[0].split('/')]
             d['runtimes'] = rt
-        country = _findBetween(cont, 'href="/Sections/Countries/', '"')
+        country = _findBetween(cont, 'href="/Sections/Countries/', ['"', '/'])
         if country: d['countries'] = country
-        lang = _findBetween(cont, 'href="/Sections/Languages/', '"')
+        lang = _findBetween(cont, 'href="/Sections/Languages/', ['"', '/'])
         if lang: d['languages'] = lang
-        col = _findBetween(cont, '"/List?color-info=', '<br>')
+        col = _findBetween(cont, '"/List?color-info=', '<br')
         if col:
             col[:] = col[0].split(' / ')
             col[:] = ['<a %s' % x for x in col if x]
             col[:] = [_unHtml(x.replace(' <i>', '::')) for x in col]
-            if col: d['color'] = col
+            if col: d['color info'] = col
         sm = _findBetween(cont, '/List?sound-mix=', '<br>')
         if sm:
             sm[:] = sm[0].split(' / ')
             sm[:] = ['<a %s' % x for x in sm if x]
             sm[:] = [_unHtml(x.replace(' <i>', '::')) for x in sm]
             if sm: d['sound mix'] = sm
-        cert = _findBetween(cont, 'Certification:</b>', '<br>')
+        cert = _findBetween(cont, 'Certification:</b>', '<br')
         if cert:
             cert[:] = cert[0].split(' / ')
             cert[:] = [_unHtml(x.replace(' <i>', '::')) for x in cert]
             if cert: d['certificates'] = cert
+        plotoutline = _findBetween(cont, 'Plot Outline:</b>', ['<a ', '<br'])
+        if plotoutline:
+            plotoutline = plotoutline[0].strip()
+            if plotoutline: d['plot outline'] = plotoutline
         return {'data': d}
 
     def get_movie_plot(self, movieID):
@@ -281,26 +302,28 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         return {'data': {}}
 
     def _search_person(self, name, results):
-        params = urllib.urlencode({'nm': 'on', 'mx': str(results), 'q': name})
+        ##params = urllib.urlencode({'nm': 'on', 'mx': str(results), 'q': name})
+        params = 'q=%s&nm=on&mx=%s' % (urllib.quote_plus(name), str(results))
         cont = self._mretrieve(imdbURL_search % params)
         name = _findBetween(cont, '<title>', '</title>')
         res = []
         if not name: return res
-        if name[0].lower().find('imdb name search') == -1:
+        nl = name[0].lower()
+        if not nl.startswith('imdb name'):
             # XXX: a direct hit!
             name = _unHtml(name[0])
             pidtag = _getTagWith(cont, '/board/threads/')
             pid = None
             if pidtag: pid = _findBetween(pidtag[0], '/name/nm', '/')
             if not (pid and name): return res
-            res[:] = [(pid[0], analyze_name(name, canonical=1))]
+            res[:] = [(str(pid[0]), analyze_name(name, canonical=1))]
         else:
-            lis = _findBetween(cont, '<li>', ['<small>', '</li>', '<br>'])
+            lis = _findBetween(cont, '<li>', ['<small', '</li>', '<br'])
             for li in lis:
                 pid = re_imdbID.findall(li)
                 pname = _unHtml(li)
                 if not (pid and pname): continue
-                res.append((pid[0], analyze_name(pname, canonical=1)))
+                res.append((str(pid[0]), analyze_name(pname, canonical=1)))
         return res
 
     def get_person_main(self, personID):
@@ -309,7 +332,8 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         name = _findBetween(s, '<title>', '</title>')
         if not name:
             raise IMDbDataAccessError, 'unable to get personID "%s"' % personID
-        r = analyze_name(name[0], canonical=1)
+        name = _unHtml(name[0])
+        r = analyze_name(name, canonical=1)
         bdate = _findBetween(s, '<div class="ch">Date of birth',
                             ('<br>', '<dt>'))
         if bdate:
@@ -337,21 +361,27 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         if hs:
             hs[:] = _findBetween(hs[0], 'src="', '"')
             if hs: r['headshot'] = hs[0]
-        workkind = _findBetween(s, 'Filmography as:</i>', '</p>')
-        if not workkind: return r
-        wsects = workkind[0].split(', ')
+        workkind = _findBetween(s, '<b><a name=', '</a> - filmography')
+        
         ws = []
-        for w in wsects:
-            sl = _findBetween(w, 'href="#', '"')
-            if not sl: continue
-            sn = _findBetween(w, '">', '</a')
-            if sn: sn = _unHtml(sn[0])
-            if not sn: continue
-            ws.append((sl[0], sn.lower()))
+        for w in workkind:
+            if w and w[0] == '"': w = w[1:]
+            se = ''
+            sn = ''
+            eti = w.rfind('>')
+            if eti == -1: continue
+            se = w[:eti]
+            if se and se[-1] == '"': se = se[:-1]
+            se = se.strip()
+            sn = w[eti+1:].strip().lower()
+            if se and sn: ws.append((se, sn))
+        if s.find('<a href="#guest-appearances"') != -1:
+            ws.append(('guest-appearances', 'notable tv guest appearances'))
+        if s.find('<a href="#archive">') != -1:
+            ws.append(('archive', 'archive footage'))
         for sect, sectName in ws:
-            sectName = sectName.lower()
             raws = ''
-            inisect = s.find('<a name="%s">' % sect)
+            inisect = s.find('<a name="%s' % sect)
             if inisect != -1:
                 endsect = s[inisect:].find('</ol>')
                 if endsect != -1: raws = s[inisect:inisect+endsect]
@@ -359,7 +389,9 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             mlist = _findBetween(raws, '<li>', ('</li>', '<br>'))
             for m in mlist:
                 d = {}
-                d['movieID'] = m[:7]
+                mid = re_imdbID.findall(m)
+                if not mid: continue
+                d['movieID'] = mid[0]
                 ti = m.find('/">')
                 te = m.find('</a>')
                 if ti != -1 and te > ti:
@@ -381,7 +413,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                     if fae != -1:
                         m = m[:fai] + m[fai+fae+4:]
                 tvi = m.find('<small>TV Series</small>')
-                if tvi != -1:
+                if tvi != -1 and d['title'][0] != '"':
                     d['title'] = '"%s"' % d['title']
                     m = m[:tvi] + m[tvi+24:]
                 m = m.strip()
@@ -393,27 +425,40 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                                     m[1:ey] in ('TV', 'V', 'mini', 'VG'):
                                 d['title'] += ' %s' % m[:ey+1]
                                 m = m[ey+1:].lstrip()
-                #istvguest = 0
-                #if m.find('<small>playing</small>') != -1:
-                #    istvguest = 1
-                m = m.replace('<small>', ' ').replace('</small>', ' ').strip()
-                notes = ''
-                role = ''
-                ms = m.split('....')
-                if len(ms) >= 1:
-                    first = ms[0]
-                    if first and first[0] == '(':
-                        notes = first.strip()
-                    ms = ms[1:]
-                if ms: role = ' '.join(ms).strip()
-                movie = Movie(title=d['title'], accessSystem=self.accessSystem,
-                                movieID=d['movieID'],
+                istvguest = 0
+                if m.find('<small>playing</small>') != -1:
+                    istvguest = 1
+                    m = m.replace('<small>playing</small>', '').strip()
+                else:
+                    m=m.replace('<small>',' ').replace('</small>',' ').strip()
+                notes = u''
+                role = u''
+                if not istvguest:
+                    ms = m.split('....')
+                    if len(ms) >= 1:
+                        first = ms[0]
+                        if first and first[0] == '(':
+                            notes = first.strip()
+                        ms = ms[1:]
+                    if ms: role = ' '.join(ms).strip()
+                else:
+                    # XXX: strip quotes from strings like "Himself"?
+                    noteidx = m.find('(')
+                    if noteidx == -1:
+                        noteidx = m.find('in episode')
+                    if noteidx != -1:
+                        notes = m[noteidx:]
+                        role = m[:noteidx-1]
+                    else:
+                        role = m
+                m_title = _unHtml(d['title'])
+                movie = Movie(title=m_title, accessSystem=self.accessSystem,
+                                movieID=str(d['movieID']),
                                 modFunct=self._defModFunct)
-                if d.has_key('status'): movie['status'] = d['status']
-                movie.currentRole = role
-                movie.notes = notes
-                if not r.has_key(sectName): r[sectName] = []
-                r[sectName].append(movie)
+                if d.has_key('status'): movie['status'] = _unHtml(d['status'])
+                if role: movie.currentRole = _unHtml(role)
+                if notes: movie.notes = _unHtml(notes)
+                r.setdefault(sectName, []).append(movie)
         return {'data': r, 'info sets': ('main', 'filmography')}
 
     def get_person_biography(self, personID):
@@ -435,8 +480,10 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         for sect, data in misc_sects:
             sect = sect.lower().replace(':', '').strip()
             if sect == 'salary': sect = 'salary history'
-            if sect in ('imdb mini-biography by', 'spouse'):
-                continue
+            elif sect in ('imdb mini-biography by', 'spouse'): continue
+            elif sect == 'nickname': sect = 'nick names'
+            elif sect == 'where are they now': sect = 'where now'
+            elif sect == 'personal quotes': sect = 'quotes'
             data = data.replace('</p><p class="biopar">', '::')
             data = data.replace('</td>\n<td valign="top">', '@@@@')
             data = data.replace('</td>\n</tr>', '::')
@@ -444,6 +491,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             data = [x.strip() for x in data.split('::')]
             data[:] = [x.replace('@@@@', '::') for x in data if x]
             if sect in ('birth name', 'height') and data: data = data[0]
+            if sect == 'birth name': data = canonicalName(data)
             d[sect] = data
         return {'data': d}
 
